import os
from sparknlp_display.ner import NerVisualizer
from sparknlp_display.dependency_parser import DependencyParserVisualizer
from sparknlp_display.relation_extraction import RelationExtractionVisualizer
from sparknlp_display.entity_resolution import EntityResolverVisualizer
from sparknlp_display.assertion import AssertionVisualizer

here = os.path.abspath(os.path.dirname(__file__))

def get_version():
    version_path = os.path.abspath(os.path.dirname(__file__))
    with open(os.path.join(here, "VERSION"), "r") as fh:
        app_version = fh.read().strip()
        return app_version

__version__ = get_version()

def version():
    return get_version()


import random
import os
import json
from . import style_utils as style_config
from IPython.display import display, HTML

here = os.path.abspath(os.path.dirname(__file__))

class AssertionVisualizer:
    def __init__(self):
        with open(os.path.join(here, 'label_colors/ner.json'), 'r', encoding='utf-8') as f_:
            self.label_colors = json.load(f_)

    #public function to get color for a label
    def get_label_color(self, label):
        """Returns color of a particular label
        
        Input: entity label <string>
        Output: Color <string> or <None> if not found
        """

        if str(label).lower() in self.label_colors:
            return self.label_colors[label.lower()]
        else:
            return None

    # private function for colors for display
    def __get_label(self, label):
        """Set label colors.
        
        Input: dictionary of entity labels and corresponding colors
        Output: self object - to allow chaining
        Note: Previous values of colors will be overwritten
        """
        if str(label).lower() in self.label_colors:
            return self.label_colors[label.lower()]
        else:
            #update it to fetch from git new labels 
            r = lambda: random.randint(0,200)
            return '#%02X%02X%02X' % (r(), r(), r())

    def set_label_colors(self, color_dict):
        """Sets label colors.

        input: dictionary of entity labels and corresponding colors
        output: self object - to allow chaining
        note: Previous values of colors will be overwritten
        """

        for key, value in color_dict.items():
          self.label_colors[key.lower()] = value
        return self      

    def __verify_structure(self, result, label_col, document_col, original_text):

        if original_text is None:
            basic_msg_1 = """Incorrect annotation structure of '{}'.""".format(document_col)
            if not hasattr(result[document_col][0], 'result'):
                raise AttributeError(basic_msg_1+""" 'result' attribute not found in the annotation. 
                Make sure '"""+document_col+"""' is a list of objects having the following structure: 
                    Annotation(type='annotation', begin=0, end=10, result='This is a text')
                Or You can pass the text manually using 'raw_text' argument.""")

        basic_msg_1 = """Incorrect annotation structure of '{}'.""".format(label_col)
        basic_msg = """
        In sparknlp please use 'LightPipeline.fullAnnotate' for LightPipeline or 'Pipeline.transform' for PipelineModel.
        Or 
        Make sure '"""+label_col+"""' is a list of objects having the following structure: 
            Annotation(type='annotation', begin=0, end=0, result='Adam', metadata={'entity': 'PERSON'})"""

        for entity in result[label_col]:
            if not hasattr(entity, 'begin'):
                raise AttributeError( basic_msg_1 + """ 'begin' attribute not found in the annotation."""+basic_msg)
            if not hasattr(entity, 'end'):
                raise AttributeError(basic_msg_1 + """ 'end' attribute not found in the annotation."""+basic_msg)
            if not hasattr(entity, 'result'):
                raise AttributeError(basic_msg_1 + """ 'result' attribute not found in the annotation."""+basic_msg)
            if not hasattr(entity, 'metadata'):
                raise AttributeError(basic_msg_1 + """ 'metadata' attribute not found in the annotation."""+basic_msg)
            if 'entity' not in entity.metadata:
                raise AttributeError(basic_msg_1+""" KeyError: 'entity' not found in metadata."""+basic_msg)

    def __verify_input(self, result, label_col, document_col, original_text):
        # check if label colum in result
        if label_col not in result:
            raise AttributeError("""column/key '{}' not found in the provided dataframe/dictionary.
            Please specify the correct key/column using 'label_col' argument.""".format(label_col))
        
        if original_text is not None:
            # check if provided text is correct data type
            if not isinstance(original_text, str):
                raise ValueError("Invalid value for argument 'raw_text' input. Text should be of type 'str'.")
        
        else:
            # check if document column in result
            if document_col not in result:
                raise AttributeError("""column/key '{}' not found in the provided dataframe/dictionary.
                Please specify the correct key/column using 'document_col' argument.
                Or You can pass the text manually using 'raw_text' argument""".format(document_col))

        self.__verify_structure( result, label_col, document_col, original_text)

    # main display function
    def __display_ner(self, result, label_col, resolution_col, document_col, original_text, labels_list = None):

        if original_text is None:
            original_text = result[document_col][0].result

        if labels_list is not None:
            labels_list = [v.lower() for v in labels_list]
            
        assertion_temp_dict = {}
        for resol in result[resolution_col]:
            assertion_temp_dict[int(resol.begin)] = resol.result
            
        label_color = {}
        html_output = ""
        pos = 0
        for entity in result[label_col]:
            entity_type = entity.metadata['entity'].lower()
            if (entity_type not in label_color) and ((labels_list is None) or (entity_type in labels_list)) :
                label_color[entity_type] = self.__get_label(entity_type)

            begin = int(entity.begin)
            end = int(entity.end)
            if pos < begin and pos < len(original_text):
                white_text = original_text[pos:begin]
                html_output += '<span class="spark-nlp-display-others" style="background-color: white">{}</span>'.format(white_text)
            pos = end+1

            if entity_type in label_color:
                
                if begin in assertion_temp_dict:
                
                    html_output += '<span class="spark-nlp-display-entity-wrapper" style="background-color: {}"><span class="spark-nlp-display-entity-name">{} </span><span class="spark-nlp-display-entity-type">{}</span><span class="spark-nlp-display-entity-resolution" style="background-color: {}">{} </span></span>'.format(
                        label_color[entity_type] + 'B3', #color
                        entity.result, #entity - chunk
                        entity.metadata['entity'], #entity - label
                        label_color[entity_type] + 'FF', #color '#D2C8C6' 
                        assertion_temp_dict[begin] # res_assertion
                    )
                else:
                    html_output += '<span class="spark-nlp-display-entity-wrapper" style="background-color: {}"><span class="spark-nlp-display-entity-name">{} </span><span class="spark-nlp-display-entity-type">{}</span></span>'.format(
                        label_color[entity_type] + 'B3', #color
                        entity.result, #entity - chunk
                        entity.metadata['entity'] #entity - label
                    )

            else:
                html_output += '<span class="spark-nlp-display-others" style="background-color: white">{}</span>'.format(entity.result)

        if pos < len(original_text):
            html_output += '<span class="spark-nlp-display-others" style="background-color: white">{}</span>'.format(original_text[pos:])

        html_output += """</div>"""

        html_output = html_output.replace("\n", "<br>")

        return html_output

    def display(self, result, label_col, assertion_col, document_col='document', raw_text=None, return_html=False, save_path=None):
        """Displays Assertion visualization. 

        Inputs:
        result -- A Dataframe or dictionary.
        label_col -- Name of the column/key containing NER annotations.
        document_col -- Name of the column/key containing text document.
        original_text -- Original text of type 'str'. If specified, it will take precedence over 'document_col' and will be used as the reference text for display.
        labels_list -- A list of labels that should be highlighted in the output. Default: Display all labels.

        Output: Visualization
        """
        
        #self.__verifyInput(result, label_col, document_col, raw_text)
        
        html_content = self.__display_ner(result, label_col, assertion_col, document_col, raw_text)
        html_content_save = style_config.STYLE_CONFIG_ENTITIES+ " "+html_content
        
        if save_path != None:
            with open(save_path, 'w') as f_:
                f_.write(html_content_save)
        
        if return_html:
            return html_content_save
        else:
            return display(HTML(html_content_save))
        


import random
import os
import json
import pandas as pd
import numpy as np
import svgwrite
from IPython.display import display, HTML

here = os.path.abspath(os.path.dirname(__file__))

class RelationExtractionVisualizer:

    def __init__(self):
        self.color_dict = {
                            "overlap" : "lightsalmon",
                            "before" : "deepskyblue",
                            "after" : "springgreen",
                            
                            "trip": "lightsalmon",
                            "trwp": "deepskyblue",
                            "trcp": "springgreen",
                            "trap": "gold",
                            "trnap": "maroon",
                            "terp": "purple",
                            "tecp": "tomato",
                            "pip" : "slategray",
                            
                            "drug-strength" : "purple",
                            "drug-frequency": "slategray",
                            "drug-form" : "deepskyblue",
                            "dosage-drug" : "springgreen",
                            "strength-drug": "maroon",
                            "drug-dosage" : "gold"
                        }
    
    def __get_color(self, l):
        r = lambda: random.randint(100,255)
        return '#%02X%02X%02X' % (r(), r(), r())

    def __size(self, text):
        return ((len(text)+1)*9.7)-5

    def __draw_line(self, dwg, s_x , s_y, e_x, e_y, d_type, color, show_relations):
        # find the a & b points
        def get_bezier_coef(points):
            # since the formulas work given that we have n+1 points
            # then n must be this:
            n = len(points) - 1

            # build coefficents matrix
            C = 4 * np.identity(n)
            np.fill_diagonal(C[1:], 1)
            np.fill_diagonal(C[:, 1:], 1)
            C[0, 0] = 2
            C[n - 1, n - 1] = 7
            C[n - 1, n - 2] = 2

            # build points vector
            P = [2 * (2 * points[i] + points[i + 1]) for i in range(n)]
            P[0] = points[0] + 2 * points[1]
            P[n - 1] = 8 * points[n - 1] + points[n]

            # solve system, find a & b
            A = np.linalg.solve(C, P)
            B = [0] * n
            for i in range(n - 1):
                B[i] = 2 * points[i + 1] - A[i + 1]
            B[n - 1] = (A[n - 1] + points[n]) / 2

            return A, B

        # returns the general Bezier cubic formula given 4 control points
        def get_cubic(a, b, c, d):
            return lambda t: np.power(1 - t, 3) * a + 3 * np.power(1 - t, 2) * t * b + 3 * (1 - t) * np.power(t, 2) * c + np.power(t, 3) * d

        # return one cubic curve for each consecutive points
        def get_bezier_cubic(points):
            A, B = get_bezier_coef(points)
            return [
                get_cubic(points[i], A[i], B[i], points[i + 1])
                for i in range(len(points) - 1)
            ]

        # evalute each cubic curve on the range [0, 1] sliced in n points
        def evaluate_bezier(points, n):
            curves = get_bezier_cubic(points)
            return np.array([fun(t) for fun in curves for t in np.linspace(0, 1, n)])


        def draw_pointer(dwg, s_x, s_y, e_x, e_y):
            size = 8
            ratio = 2
            fullness1 = 2
            fullness2 = 3
            bx = e_x
            ax = s_x
            by = e_y
            ay = s_y
            abx = bx - ax
            aby = by - ay
            ab = np.sqrt(abx * abx + aby * aby)

            cx = bx - size * abx / ab
            cy = by - size * aby / ab
            dx = cx + (by - cy) / ratio
            dy = cy + (cx - bx) / ratio
            ex = cx - (by - cy) / ratio
            ey = cy - (cx - bx) / ratio
            fx = (fullness1 * cx + bx) / fullness2
            fy = (fullness1 * cy + by) / fullness2

            text_place_y = s_y-(abs(s_y-e_y)/2)
            line = dwg.add(dwg.polyline(
                      [
                      (bx, by),    
                      (dx, dy),
                      (fx, fy),
                      (ex, ey),
                      (bx, by)
                      ],
                      stroke=color, stroke_width = "2", fill='none',))
            return text_place_y

        if s_x > e_x:
            #s_x -= 5
            e_x += 10
        else:
            #s_x += 5
            e_x -= 2
        if s_y == e_y:
            s_y -= 20
            e_y = s_y-4#55
            text_place_y = s_y-45

            pth = evaluate_bezier(np.array([[s_x, s_y], 
                                [(s_x+e_x)/2.0, s_y-40],
                                [e_x,e_y]]), 50)
            dwg.add(dwg.polyline(pth,
                stroke=color, stroke_width = "2", fill='none',))
            
            draw_pointer(dwg, (s_x+e_x)/2.0, s_y-50, e_x, e_y)

        elif s_y >= e_y:
            e_y +=15
            s_y-=20
            dwg.add(dwg.polyline([(s_x,s_y), (e_x, e_y)],
                stroke=color, stroke_width = "2", fill='none',))
            text_place_y = draw_pointer(dwg, s_x, s_y, e_x, e_y)
        else:
            s_y-=5
            e_y -= 40
            dwg.add(dwg.polyline([(s_x,s_y), (e_x, e_y)],
                stroke=color, stroke_width = "2", fill='none',))
            text_place_y = draw_pointer(dwg, s_x, s_y, e_x, e_y)
        if show_relations:
            dwg.add(dwg.text(d_type, insert=(((s_x+e_x)/2)-(self.__size(d_type)/2.75), text_place_y), 
            fill=color, font_size='12', font_family='courier'))

    def __gen_graph(self, rdf, selected_text, show_relations):
        
        done_ent1 = {}
        done_ent2 = {}
        all_done = {}
        
        start_y = 75
        x_limit = 920
        y_offset = 100
        dwg = svgwrite.Drawing("temp.svg",profile='tiny', size = (x_limit, len(selected_text) * 1.1 + len(rdf)*20))
        
        begin_index = 0
        start_x = 10
        this_line = 0

        all_entities_index = set()
        all_entities_1_index = []
        basic_dict = {}
        relation_dict = {}
        for t in rdf:
            if t.result.lower().strip() != 'o':
                all_entities_index.add(int(t.metadata['entity1_begin']))
                all_entities_index.add(int(t.metadata['entity2_begin']))
                basic_dict[int(t.metadata['entity1_begin'])] = [t.metadata['entity1_begin'],
                                                    t.metadata['entity1_end'],
                                                    t.metadata['chunk1'], 
                                                    t.metadata['entity1']]

                basic_dict[int(t.metadata['entity2_begin'])] = [t.metadata['entity2_begin'],
                                                    t.metadata['entity2_end'],
                                                    t.metadata['chunk2'], 
                                                    t.metadata['entity2']]
            
            #all_entities_1_index.append(t[4]['entity1_begin'])
        all_entities_index = np.asarray(list(all_entities_index))
        all_entities_index = all_entities_index[np.argsort(all_entities_index)]
        for ent_start_ind in all_entities_index:
            e_start_now, e_end_now, e_chunk_now, e_entity_now = basic_dict[ent_start_ind]
            prev_text = selected_text[begin_index:int(e_start_now)]
            begin_index = int(e_end_now)+1
            for word_ in prev_text.split(' '):
                this_size = self.__size(word_)
                if (start_x + this_size + 10) >= x_limit:
                    start_y += y_offset
                    start_x = 10
                    this_line = 0
                dwg.add(dwg.text(word_, insert=(start_x, start_y ), fill='gray', font_size='16', font_family='courier'))
                start_x += this_size + 5
                
            this_size = self.__size(e_chunk_now)
            if (start_x + this_size + 10)>= x_limit:# or this_line >= 2:
                    start_y += y_offset
                    start_x = 10
                    this_line = 0
            #chunk1
            dwg.add(dwg.text(e_chunk_now, insert=(start_x, start_y ), fill='gray', font_size='16', font_family='courier'))
            #rectange chunk 1
            dwg.add(dwg.rect(insert=(start_x-3, start_y-18), size=(this_size,25), 
            rx=2, ry=2, stroke='orange', 
            stroke_width='2', fill='none'))
            #entity 1
            central_point_x = start_x+(this_size/2)
            
            dwg.add(dwg.text(e_entity_now, 
                            insert=(central_point_x-(self.__size(e_entity_now)/2.75), start_y+20), 
                            fill='slateblue', font_size='12', font_family='courier'))
            
            all_done[int(e_start_now)] = [central_point_x-(self.__size(e_entity_now)/2.75), start_y]
            start_x += this_size + 10
            this_line += 1 
              
            #all_done[ent_start_ind] = 
        
        prev_text = selected_text[begin_index:]        
        for word_ in prev_text.split(' '):
            this_size = self.__size(word_)
            if (start_x + this_size)>= x_limit:
                start_y += y_offset
                start_x = 10
            dwg.add(dwg.text(word_, insert=(start_x, start_y ), fill='gray', font_size='16', font_family='courier'))
            start_x += this_size

        for row in rdf:
            if row.result.lower().strip() != 'o':
                if row.result.lower().strip() not in self.color_dict:
                    self.color_dict[row.result.lower().strip()] = self.__get_color(row.result.lower().strip())
                d_key2 = all_done[int(row.metadata['entity2_begin'])]
                d_key1 = all_done[int(row.metadata['entity1_begin'])]
                self.__draw_line(dwg, d_key2[0] , d_key2[1], d_key1[0], d_key1[1], 
                          row.result,self.color_dict[row.result.lower().strip()], show_relations)
            
        return dwg.tostring()

    def display(self, result, relation_col, document_col='document', show_relations=True):

        original_text = result[document_col][0].result
        res = result[relation_col]
        return display(HTML(self.__gen_graph(res, original_text, show_relations)))




import random
import os
import json
import pandas as pd
import numpy as np
import svgwrite
import math
import re
from IPython.display import display, HTML

here = os.path.abspath(os.path.dirname(__file__))
#overlap_hist = []
#y_hist_dict = {}
x_i_diff_dict = {}
x_o_diff_dict = {}
class RelationExtractionVisualizer:
    
    def __init__(self):
        with open(os.path.join(here, 'label_colors/relations.json'), 'r', encoding='utf-8') as f_:
            self.color_dict = json.load(f_)
        with open(os.path.join(here, 'label_colors/ner.json'), 'r', encoding='utf-8') as f_:
            self.entity_color_dict = json.load(f_)
            self.entity_color_dict = dict((k.lower(), v) for k, v in self.entity_color_dict.items())
            self.font_path = os.path.join(here, 'fonts/Lucida_Console.ttf')
            self.main_font = 'Lucida'
    def __get_color(self, l):
        r = lambda: random.randint(0,200)
        return '#%02X%02X%02X' % (r(), r(), r())

    def __size(self, text):
        return ((len(text)+1)*9.7)-5

    def __draw_line(self, dwg, s_x , s_y, e_x, e_y, d_type, color, show_relations, size_of_entity_label):
        eps = 0.0000001
        def get_bezier_coef(points):
            # since the formulas work given that we have n+1 points
            # then n must be this:
            n = len(points) - 1

            # build coefficents matrix
            C = 4 * np.identity(n)
            np.fill_diagonal(C[1:], 1)
            np.fill_diagonal(C[:, 1:], 1)
            C[0, 0] = 2
            C[n - 1, n - 1] = 7
            C[n - 1, n - 2] = 2

            # build points vector
            P = [2 * (2 * points[i] + points[i + 1]) for i in range(n)]
            P[0] = points[0] + 2 * points[1]
            P[n - 1] = 8 * points[n - 1] + points[n]

            # solve system, find a & b
            A = np.linalg.solve(C, P)
            B = [0] * n
            for i in range(n - 1):
                B[i] = 2 * points[i + 1] - A[i + 1]
            B[n - 1] = (A[n - 1] + points[n]) / 2

            return A, B

        # returns the general Bezier cubic formula given 4 control points
        def get_cubic(a, b, c, d):
            return lambda t: np.power(1 - t, 3) * a + 3 * np.power(1 - t, 2) * t * b + 3 * (1 - t) * np.power(t, 2) * c + np.power(t, 3) * d

        # return one cubic curve for each consecutive points
        def get_bezier_cubic(points):
            A, B = get_bezier_coef(points)
            return [
                get_cubic(points[i], A[i], B[i], points[i + 1])
                for i in range(len(points) - 1)
            ]

        # evalute each cubic curve on the range [0, 1] sliced in n points
        def evaluate_bezier(points, n):
            curves = get_bezier_cubic(points)
            return np.array([fun(t) for fun in curves for t in np.linspace(0, 1, n)])


        def draw_pointer(dwg, s_x, s_y, e_x, e_y):
            size = 5
            ratio = 1
            fullness1 = 2
            fullness2 = 3
            bx = e_x
            ax = s_x
            by = e_y
            ay = s_y
            abx = bx - ax
            aby = by - ay
            ab = np.sqrt(abx * abx + aby * aby) + eps

            cx = bx - size * abx / ab
            cy = by - size * aby / ab
            dx = cx + (by - cy) / ratio
            dy = cy + (cx - bx) / ratio
            ex = cx - (by - cy) / ratio
            ey = cy - (cx - bx) / ratio
            fx = (fullness1 * cx + bx) / fullness2
            fy = (fullness1 * cy + by) / fullness2

            text_place_y = s_y-(abs(s_y-e_y)/2)
            '''
            line = dwg.add(dwg.polyline(
                      [
                      (bx, by),    
                      (dx, dy),
                      (fx, fy),
                      (ex, ey),
                      (bx, by)
                      ],
                      stroke=color, stroke_width = "1", fill='none',))
            '''
            line = dwg.add(dwg.polyline(
                      [
                      (dx, dy),
                      (bx, by),    
                      (ex, ey),
                      (bx, by)
                      ],
                      stroke=color, stroke_width = "1", fill='none',))
            return text_place_y
        unique_o_index = str(s_x)+str(s_y)
        unique_i_index = str(e_x)+str(e_y)
        if s_x > e_x:
            if unique_o_index in x_o_diff_dict:
                s_x -= 5
            else:
                s_x -= 10
                x_o_diff_dict[unique_o_index] = 5
            if s_y > e_y:
                e_x += size_of_entity_label
            
            if unique_i_index in x_i_diff_dict:
                e_x += 5
            else:
                e_x += 10
                x_i_diff_dict[unique_i_index] = 5
        else:
            if unique_o_index in x_o_diff_dict:
                s_x += 5
            else:
                s_x += 10
                x_o_diff_dict[unique_o_index] = 5
            if s_y > e_y:
                e_x -= size_of_entity_label
            if unique_i_index in x_i_diff_dict:
                e_x -= 5
            else:
                e_x -= 10
                x_i_diff_dict[unique_i_index] = 5
        #this_y_vals = list(range(min(s_x,e_x), max(s_x,e_x)+1))
        #this_y_vals = [ str(s_y)+'|'+str(i) for i in this_y_vals]
        #common = set(this_y_vals) & set(overlap_hist)
        #overlap_hist.extend(this_y_vals)
        #if s_y not in y_hist_dict:
        #    y_hist_dict[s_y] = 20
        #if common:
        #    y_hist_dict[s_y] += 20
        #y_increase = y_hist_dict[s_y]
        angle = -1
        if s_y == e_y:
            angle = 0
            s_y -= 20
            e_y = s_y-4#55
            
            text_place_y = s_y-35

            pth = evaluate_bezier(np.array([[s_x, s_y], 
                                [(s_x+e_x)/2.0, s_y-40],
                                [e_x,e_y]]), 50)
            dwg.add(dwg.polyline(pth,
                stroke=color, stroke_width = "1", fill='none',))
            draw_pointer(dwg, (s_x+e_x)/2.0, s_y-50, e_x, e_y)
        elif s_y >= e_y:
            e_y +=15
            s_y-=20
            text_place_y = s_y-(abs(s_y-e_y)/2)
            
            pth = evaluate_bezier(np.array([[s_x, s_y],
                                #[((3*s_x)+e_x)/4.0, (s_y+e_y)/2.0],
                                [(s_x+e_x)/2.0, (s_y+e_y)/2.0],
                                #[(s_x+(3*e_x))/4.0,(s_y+e_y)/2.0],
                                [e_x,e_y]]), 50)
            dwg.add(dwg.polyline(pth,
                stroke=color, stroke_width = "1", fill='none',))
            draw_pointer(dwg, s_x, s_y, e_x, e_y)

            '''
            line = dwg.add(dwg.polyline(
                    [(s_x, s_y),(s_x, s_y-y_increase), (e_x, s_y-y_increase),
                    (e_x, e_y),    
                    (e_x+2, e_y),
                    (e_x, e_y-4),
                    (e_x-2, e_y),
                    (e_x, e_y)
                    ],
                    stroke=color, stroke_width = "2", fill='none',))
            '''
        else:
            s_y-=5
            e_y -= 40
            text_place_y = s_y+(abs(s_y-e_y)/2)
            
            line = dwg.add(dwg.polyline(
                    [(s_x, s_y),
                    (e_x, e_y-40),    
                    (e_x+2, e_y),
                    (e_x, e_y+4),
                    (e_x-2, e_y),
                    (e_x, e_y)
                    ],
                    stroke=color, stroke_width = "1", fill='none',))
            draw_pointer(dwg, s_x, s_y, e_x, e_y)
            
        if show_relations:
            if angle == -1: angle = math.degrees(math.atan((s_y-e_y)/((s_x-e_x)+eps)))
            rel_temp_size = self.__size(d_type)/1.35
            rect_x, rect_y = (((s_x+e_x)/2.0)-(rel_temp_size/2.0)-3, text_place_y-10)
            rect_w, rect_h = (rel_temp_size+3,13)
            dwg.add(dwg.rect(insert=(rect_x, rect_y), rx=2,ry=2, 
            size=(rect_w, rect_h), 
            fill='white', stroke=color , stroke_width='1', 
            transform = f"rotate({angle} {rect_x+rect_w/2} {rect_y+rect_h/2})"))

            dwg.add(dwg.text(d_type, insert=(((s_x+e_x)/2)-(rel_temp_size/2.0), text_place_y), 
            fill=color, font_size='12', font_family='courier',
            transform = f"rotate({angle} {rect_x+rect_w/2} {rect_y+rect_h/2})"))

    def __gen_graph(self, rdf, selected_text, exclude_relations, show_relations):
        exclude_relations = [ i.lower().strip() for i in exclude_relations]
        rdf = [ i for i in rdf if i.result.lower().strip() not in exclude_relations]

        done_ent1 = {}
        done_ent2 = {}
        all_done = {}
        
        start_y = 75
        x_limit = 1000
        y_offset = 100
        #dwg = svgwrite.Drawing("temp.svg",profile='full', size = (x_limit, len(selected_text) * 1.1 + len(rdf)*20))
        
        begin_index = 0
        start_x = 10
        this_line = 0

        all_entities_index = set()
        all_entities_1_index = []
        basic_dict = {}
        relation_dict = {}
        for t in rdf:
        
            all_entities_index.add(int(t.metadata['entity1_begin']))
            all_entities_index.add(int(t.metadata['entity2_begin']))
            basic_dict[int(t.metadata['entity1_begin'])] = [t.metadata['entity1_begin'],
                                                t.metadata['entity1_end'],
                                                t.metadata['chunk1'], 
                                                t.metadata['entity1']]

            basic_dict[int(t.metadata['entity2_begin'])] = [t.metadata['entity2_begin'],
                                                t.metadata['entity2_end'],
                                                t.metadata['chunk2'], 
                                                t.metadata['entity2']]
            if t.metadata['entity1'].lower().strip() not in self.entity_color_dict:
                self.entity_color_dict[t.metadata['entity1'].lower().strip()] = self.__get_color(t.metadata['entity1'].lower().strip())
            if t.metadata['entity2'].lower().strip() not in self.entity_color_dict:
                self.entity_color_dict[t.metadata['entity2'].lower().strip()] = self.__get_color(t.metadata['entity2'].lower().strip())  
             
        
            #all_entities_1_index.append(t[4]['entity1_begin'])
        all_entities_index = np.asarray(list(all_entities_index))
        all_entities_index = all_entities_index[np.argsort(all_entities_index)]
        dwg_rects, dwg_texts = [], []
        for ent_start_ind in all_entities_index:
            e_start_now, e_end_now, e_chunk_now, e_entity_now = basic_dict[ent_start_ind]
            prev_text = selected_text[begin_index:int(e_start_now)]
            prev_text = re.sub(r'\s*(\n)+', r'\1', prev_text.strip(), re.MULTILINE)
            begin_index = int(e_end_now)+1
            for line_num, line in enumerate(prev_text.split('\n')):
                if line_num != 0:
                    start_y += y_offset
                    start_x = 10
                    this_line = 0
                for word_ in line.split(' '):
                    this_size = self.__size(word_)
                    if (start_x + this_size + 10) >= x_limit:
                        start_y += y_offset
                        start_x = 10
                        this_line = 0
                    dwg_texts.append([word_, (start_x, start_y ), '#546c74', '16', self.main_font, 'font-weight:100'])
                    #dwg.add(dwg.text(word_, insert=(start_x, start_y ), fill='#546c77', font_size='16', 
                    #                 font_family='Monaco', style='font-weight:lighter'))
                    start_x += this_size + 10
                
            this_size = self.__size(e_chunk_now)
            if (start_x + this_size + 10)>= x_limit:# or this_line >= 2:
                    start_y += y_offset
                    start_x = 10
                    this_line = 0
                    
            #rectange chunk 1
            dwg_rects.append([(start_x-3, start_y-18), (this_size,25), self.entity_color_dict[e_entity_now.lower().strip()]])
            #dwg.add(dwg.rect(insert=(start_x-3, start_y-18),rx=2,ry=2, size=(this_size,25), stroke=self.entity_color_dict[e_entity_now.lower()], 
            #stroke_width='1', fill=self.entity_color_dict[e_entity_now.lower()], fill_opacity='0.2'))
            #chunk1
            dwg_texts.append([e_chunk_now, (start_x, start_y ), '#546c74', '16', self.main_font, 'font-weight:100'])
            #dwg.add(dwg.text(e_chunk_now, insert=(start_x, start_y ), fill='#546c77', font_size='16', 
            #                 font_family='Monaco', style='font-weight:lighter'))
            #entity 1
            central_point_x = start_x+(this_size/2)
            temp_size = self.__size(e_entity_now)/2.75
            dwg_texts.append([e_entity_now.upper(), (central_point_x-temp_size, start_y+20), '#1f77b7', '12', self.main_font, 'font-weight:lighter'])
            #dwg.add(dwg.text(e_entity_now.upper(), 
            #                insert=(central_point_x-temp_size, start_y+20), 
            #                fill='#1f77b7', font_size='12', font_family='Monaco',
            #                style='font-weight:lighter'))
            
            all_done[int(e_start_now)] = [central_point_x, start_y, temp_size]
            start_x += this_size + 20
            this_line += 1 
        

        prev_text = selected_text[begin_index:]
        prev_text = re.sub(r'\s*(\n)+', r'\1', prev_text.strip(), re.MULTILINE)
        for line_num, line in enumerate(prev_text.split('\n')):
            if line_num != 0:
                start_y += y_offset
                start_x = 10
            for word_ in line.split(' '):
                this_size = self.__size(word_)
                if (start_x + this_size)>= x_limit:
                    start_y += y_offset
                    start_x = 10
                dwg_texts.append([word_, (start_x, start_y ), '#546c77', '16', self.main_font, 'font-weight:100'])
                #dwg.add(dwg.text(word_, insert=(start_x, start_y ), fill='#546c77', font_size='16', 
                #                 font_family='Monaco', style='font-weight:lighter'))
                start_x += this_size + 10
            
        
        dwg = svgwrite.Drawing("temp.svg",profile='full', size = (x_limit, start_y+y_offset))
        dwg.embed_font(self.main_font, self.font_path)
        
        for crect_ in dwg_rects:
            dwg.add(dwg.rect(insert=crect_[0],rx=2,ry=2, size=crect_[1], stroke=crect_[2], 
            stroke_width='1', fill=crect_[2], fill_opacity='0.2'))
            
        for ctext_ in dwg_texts:
            dwg.add(dwg.text(ctext_[0], insert=ctext_[1], fill=ctext_[2], font_size=ctext_[3], 
                             font_family=ctext_[4], style=ctext_[5]))
        
        
        relation_distances = []
        relation_coordinates = []
        for row in rdf:
            if row.result.lower().strip() not in self.color_dict:
                self.color_dict[row.result.lower().strip()] = self.__get_color(row.result.lower().strip())
            
            d_key2 = all_done[int(row.metadata['entity2_begin'])]
            d_key1 = all_done[int(row.metadata['entity1_begin'])]
            this_dist = abs(d_key2[0] - d_key1[0]) + abs (d_key2[1]-d_key1[1])
            relation_distances.append(this_dist)
            relation_coordinates.append((d_key2, d_key1, row.result))
        
        relation_distances = np.array(relation_distances)
        relation_coordinates = np.array(relation_coordinates)
        temp_ind = np.argsort(relation_distances)
        relation_distances = relation_distances[temp_ind]
        relation_coordinates = relation_coordinates[temp_ind]
        for row in relation_coordinates:
            #if int(row[0][1]) == int(row[1][1]):
            size_of_entity_label = int(row[1][2])
            self.__draw_line(dwg, int(row[0][0]) , int(row[0][1]), int(row[1][0]), int(row[1][1]), 
                            row[2],self.color_dict[row[2].lower().strip()], show_relations, size_of_entity_label)
        
        return dwg.tostring()

    def display(self, result, relation_col, document_col='document', exclude_relations=['O'], show_relations=True, return_html=False, save_path=None):
        """Displays Relation Extraction visualization. 
        Inputs:
        result -- A Dataframe or dictionary.
        relation_col -- Name of the column/key containing relationships.
        document_col -- Name of the column/key containing text document.
        exclude_relations -- list of relations that don't need to be displayed. Default: ["O"]
        show_relations -- Display relation types on arrows. Default: True
        return_html -- If true, returns raw html code instead of displaying. Default: False
        Output: Visualization
        """

        original_text = result[document_col][0].result
        res = result[relation_col]
        
        html_content = self.__gen_graph(res, original_text, exclude_relations, show_relations)
        
        if save_path != None:
            with open(save_path, 'w') as f_:
                f_.write(html_content)
        
        if return_html:
            return html_content
        else:
            return display(HTML(html_content))



import random
import os
import json
import pandas as pd
import numpy as np
import svgwrite
from IPython.display import display, HTML

here = os.path.abspath(os.path.dirname(__file__))
overlap_hist = []
y_hist_dict = {}
x_i_diff_dict = {}
x_o_diff_dict = {}
class RelationExtractionVisualizer:
    
    def __init__(self):
        self.color_dict = {
                            "overlap" : "lightsalmon",
                            "before" : "deepskyblue",
                            "after" : "springgreen",
                            
                            "trip": "lightsalmon",
                            "trwp": "deepskyblue",
                            "trcp": "springgreen",
                            "trap": "gold",
                            "trnap": "maroon",
                            "terp": "purple",
                            "tecp": "tomato",
                            "pip" : "slategray",
                            
                            "drug-strength" : "purple",
                            "drug-frequency": "slategray",
                            "drug-form" : "deepskyblue",
                            "dosage-drug" : "springgreen",
                            "strength-drug": "maroon",
                            "drug-dosage" : "gold"
                        }
            
    def __get_color(self, l):
        r = lambda: random.randint(100,255)
        return '#%02X%02X%02X' % (r(), r(), r())

    def __size(self, text):
        return ((len(text)+1)*9.7)-5

    def __draw_line(self, dwg, s_x , s_y, e_x, e_y, d_type, color, show_relations):
        if s_x > e_x:
            if s_x in x_o_diff_dict:
              x_o_diff_dict[s_x] -= 10
            else:
              x_o_diff_dict[s_x] = 10
            if e_x in x_i_diff_dict:
              x_i_diff_dict[e_x] += 10
            else:
              x_i_diff_dict[e_x] = 10
            s_x -= x_o_diff_dict[s_x]
            e_x += x_i_diff_dict[e_x]
        else:
            if s_x in x_o_diff_dict:
              x_o_diff_dict[s_x] += 10
            else:
              x_o_diff_dict[s_x] = 10
            if e_x in x_i_diff_dict:
              x_i_diff_dict[e_x] -= 10
            else:
              x_i_diff_dict[e_x] = 10
            s_x += x_o_diff_dict[s_x]
            e_x -= x_i_diff_dict[e_x]
        this_y_vals = list(range(min(s_x,e_x), max(s_x,e_x)+1))
        this_y_vals = [ str(s_y)+'|'+str(i) for i in this_y_vals]
        common = set(this_y_vals) & set(overlap_hist)
        overlap_hist.extend(this_y_vals)
        if s_y not in y_hist_dict:
            y_hist_dict[s_y] = 20
        if common:
            y_hist_dict[s_y] += 20
        y_increase = y_hist_dict[s_y]
        if s_y == e_y:
            s_y -= 20
            e_y = s_y-4#55
            
            text_place_y = s_y-45

            line = dwg.add(dwg.polyline(
                [(s_x, s_y), (s_x, s_y-y_increase), (e_x, s_y-y_increase),
                (e_x, e_y),    
                (e_x+2, e_y),
                (e_x, e_y+4),
                (e_x-2, e_y),
                (e_x, e_y)
                ],
                stroke=color, stroke_width = "2", fill='none',))
        elif s_y >= e_y:
            e_y +=30
            s_y-=20
            text_place_y = s_y-(abs(s_y-e_y)/2)
            line = dwg.add(dwg.polyline(
                    [(s_x, s_y),(s_x, s_y-y_increase), (e_x, s_y-y_increase),
                    (e_x, e_y),    
                    (e_x+2, e_y),
                    (e_x, e_y-4),
                    (e_x-2, e_y),
                    (e_x, e_y)
                    ],
                    stroke=color, stroke_width = "2", fill='none',))
        else:
            s_y-=5
            e_y -= 40
            text_place_y = s_y+(abs(s_y-e_y)/2)
            line = dwg.add(dwg.polyline(
                    [(s_x, s_y),
                    (e_x, e_y-40),    
                    (e_x+2, e_y),
                    (e_x, e_y+4),
                    (e_x-2, e_y),
                    (e_x, e_y)
                    ],
                    stroke=color, stroke_width = "2", fill='none',))
        if show_relations:
            dwg.add(dwg.text(d_type, insert=(((s_x+e_x)/2)-(self.__size(d_type)/2.75), text_place_y), 
            fill=color, font_size='12', font_family='courier'))

    def __gen_graph(self, rdf, selected_text, show_relations):
        rdf = [ i for i in rdf if i.result.lower().strip()!='o']

        done_ent1 = {}
        done_ent2 = {}
        all_done = {}
        
        start_y = 75
        x_limit = 920
        y_offset = 100
        dwg = svgwrite.Drawing("temp.svg",profile='tiny', size = (x_limit, len(selected_text) * 1.1 + len(rdf)*20))
        
        begin_index = 0
        start_x = 10
        this_line = 0

        all_entities_index = set()
        all_entities_1_index = []
        basic_dict = {}
        relation_dict = {}
        for t in rdf:
        
            all_entities_index.add(int(t.metadata['entity1_begin']))
            all_entities_index.add(int(t.metadata['entity2_begin']))
            basic_dict[int(t.metadata['entity1_begin'])] = [t.metadata['entity1_begin'],
                                                t.metadata['entity1_end'],
                                                t.metadata['chunk1'], 
                                                t.metadata['entity1']]

            basic_dict[int(t.metadata['entity2_begin'])] = [t.metadata['entity2_begin'],
                                                t.metadata['entity2_end'],
                                                t.metadata['chunk2'], 
                                                t.metadata['entity2']]
        
            #all_entities_1_index.append(t[4]['entity1_begin'])
        all_entities_index = np.asarray(list(all_entities_index))
        all_entities_index = all_entities_index[np.argsort(all_entities_index)]
        for ent_start_ind in all_entities_index:
            e_start_now, e_end_now, e_chunk_now, e_entity_now = basic_dict[ent_start_ind]
            prev_text = selected_text[begin_index:int(e_start_now)]
            begin_index = int(e_end_now)+1
            for word_ in prev_text.split(' '):
                this_size = self.__size(word_)
                if (start_x + this_size + 10) >= x_limit:
                    start_y += y_offset
                    start_x = 10
                    this_line = 0
                dwg.add(dwg.text(word_, insert=(start_x, start_y ), fill='gray', font_size='16', font_family='courier'))
                start_x += this_size + 5
                
            this_size = self.__size(e_chunk_now)
            if (start_x + this_size + 10)>= x_limit:# or this_line >= 2:
                    start_y += y_offset
                    start_x = 10
                    this_line = 0
            #chunk1
            dwg.add(dwg.text(e_chunk_now, insert=(start_x, start_y ), fill='gray', font_size='16', font_family='courier'))
            #rectange chunk 1
            dwg.add(dwg.rect(insert=(start_x-3, start_y-18),rx=3,ry=3, size=(this_size,25), stroke='orange', 
            stroke_width='2', fill='none'))
            #entity 1
            central_point_x = start_x+(this_size/2)
            
            dwg.add(dwg.text(e_entity_now, 
                            insert=(central_point_x-(self.__size(e_entity_now)/2.75), start_y+20), 
                            fill='mediumseagreen', font_size='12', font_family='courier'))
            
            all_done[int(e_start_now)] = [central_point_x, start_y]
            start_x += this_size + 10
            this_line += 1 
              
            #all_done[ent_start_ind] = 
        
        prev_text = selected_text[begin_index:]        
        for word_ in prev_text.split(' '):
            this_size = self.__size(word_)
            if (start_x + this_size)>= x_limit:
                start_y += y_offset
                start_x = 10
            dwg.add(dwg.text(word_, insert=(start_x, start_y ), fill='gray', font_size='16', font_family='courier'))
            start_x += this_size

        relation_distances = []
        relation_coordinates = []
        for row in rdf:
            if row.result.lower().strip() not in self.color_dict:
                self.color_dict[row.result.lower().strip()] = self.__get_color(row.result.lower().strip())
            d_key2 = all_done[int(row.metadata['entity2_begin'])]
            d_key1 = all_done[int(row.metadata['entity1_begin'])]
            this_dist = abs(d_key2[0] - d_key1[0]) + abs (d_key2[1]-d_key1[1])
            relation_distances.append(this_dist)
            relation_coordinates.append((d_key2, d_key1, row.result))
        
        relation_distances = np.array(relation_distances)
        relation_coordinates = np.array(relation_coordinates)
        temp_ind = np.argsort(relation_distances)
        relation_distances = relation_distances[temp_ind]
        relation_coordinates = relation_coordinates[temp_ind]
        for row in relation_coordinates:
            self.__draw_line(dwg, int(row[0][0]) , int(row[0][1]), int(row[1][0]), int(row[1][1]), 
                            row[2],self.color_dict[row[2].lower().strip()], show_relations)
        
        return dwg.tostring()

    def display(self, result, relation_col, document_col='document', show_relations=True):

        original_text = result[document_col][0].result
        res = result[relation_col]
        return display(HTML(self.__gen_graph(res, original_text, show_relations)))



import os

HTML_WRAPPER = """<div class="spark-nlp-display-scroll-entities" style="overflow-x: auto; border: 1px solid #e6e9ef; border-radius: 0.25rem; padding: 1rem; margin-bottom: 2.5rem; white-space:pre-wrap">{}</div>"""
HTML_INDEX_WRAPPER = """<div ">{}</div>"""

STYLE_CONFIG_ENTITIES = f"""
<style>
    @import url('https://fonts.googleapis.com/css2?family=Montserrat:wght@300;400;500;600;700&display=swap');
    @import url('https://fonts.googleapis.com/css2?family=Vistol Regular:wght@300;400;500;600;700&display=swap');
    
    .spark-nlp-display-scroll-entities {{
        border: 1px solid #E7EDF0;
        border-radius: 3px;
        text-align: justify;
        
    }}
    .spark-nlp-display-scroll-entities span {{  
        font-size: 14px;
        line-height: 24px;
        color: #536B76;
        font-family: 'Montserrat', sans-serif !important;
    }}
    
    .spark-nlp-display-entity-wrapper{{
    
        display: inline-grid;
        text-align: center;
        border-radius: 4px;
        margin: 0 2px 5px 2px;
        padding: 1px
    }}
    .spark-nlp-display-entity-name{{
        font-size: 14px;
        line-height: 24px;
        font-family: 'Montserrat', sans-serif !important;
        
        background: #f1f2f3;
        border-width: medium;
        text-align: center;
        
        font-weight: 400;
        
        border-radius: 5px;
        padding: 2px 5px;
        display: block;
        margin: 3px 2px;
    
    }}
    .spark-nlp-display-entity-type{{
        font-size: 14px;
        line-height: 24px;
        color: #ffffff;
        font-family: 'Montserrat', sans-serif !important;
        
        text-transform: uppercase;
        
        font-weight: 500;

        display: block;
        padding: 3px 5px;
    }}
    
    .spark-nlp-display-entity-resolution{{
        font-size: 14px;
        line-height: 24px;
        color: #ffffff;
        font-family: 'Vistol Regular', sans-serif !important;
        
        text-transform: uppercase;
        
        font-weight: 500;

        display: block;
        padding: 3px 5px;
    }}
    
    .spark-nlp-display-others{{
        font-size: 14px;
        line-height: 24px;
        font-family: 'Montserrat', sans-serif !important;
        
        font-weight: 400;
    }}

</style>
"""



import random
import os
import json
import pandas as pd
import numpy as np
import svgwrite
from IPython.display import display, HTML

here = os.path.abspath(os.path.dirname(__file__))

class DependencyParserVisualizer:

    def __get_color(self, l):
        r = lambda: random.randint(100,255)
        return '#%02X%02X%02X' % (r(), r(), r())

    def __size(self, text):
        return ((len(text)+1)*15)-5

    def __draw_line(self, dwg, s_x , s_y, e_x, e_y, d_type, color):
      rx=ry=1
      dwg.add(dwg.path(d=f"M{min(s_x,e_x)},{s_y} A {rx}, {ry}, 0 1 1 {max(s_x,e_x)}, {s_y}",
      stroke=color, stroke_width = "3", fill='none'))
      dwg.add(dwg.polyline(
          [(e_x, s_y), (e_x+3, s_y),
          (e_x, s_y+5),
          (e_x-3, s_y),
          (e_x, s_y)
          ], stroke=color, stroke_width = "4", fill='none',))
      '''
      line = dwg.add(dwg.polyline(
          [(s_x, s_y+4), 
          (s_x, e_y), 
          (e_x, e_y), 
          (e_x, s_y),
          (e_x+2, s_y),
          (e_x, s_y+4),
          (e_x-2, s_y),
          (e_x, s_y)
          ],
          stroke=color, stroke_width = "2", fill='none',))
      '''
      dwg.add(dwg.text(d_type, insert=(((s_x+e_x)/2)-(self.__size(d_type.strip())/2.75), e_y-4), 
      fill=color, font_size='20', font_family='courier'))

    def __generate_graph(self, result_df):
        # returns an svg graph

        colors_dict = {}
        max_x = 50
        max_y = 100
        
        for i in result_df['dependency_type'].unique():
            colors_dict[i] = self.__get_color(i)
        
        for i in result_df['pos'].unique():
            colors_dict[i] = self.__get_color(i)

        for i, row in result_df.iterrows():
            txt = row['chunk'].strip()
            max_x += (self.__size(txt) + 50)
            max_y += 30
        
        max_x += 50
        start_x = 50
        starty_y = max_y
        dp_dict={}
        tk_dict = {}
        dist_dict = {}
        main_text = []
        main_pos = []

        for i, row in result_df.iterrows():
            txt = row['chunk'].strip()
            dt = row['dependency_type'].lower().strip()
            is_root = False
            if dt == 'root':
              is_root = True
            main_text.append((txt, start_x, starty_y, is_root))
            main_pos.append((row['pos'].strip(), (start_x + int((self.__size(txt)/2) - int(self.__size(row['pos'])/2))), starty_y+30))
            
            tk_dict[str(row['begin'])+str(row['end'])] = (start_x+int(self.__size(txt)/2), starty_y)
            start_x += (self.__size(txt) + 50)
        
        y_offset = starty_y-100
        dist_dict = {}
        e_dist_dict = {}
        direct_dict = {}
        left_side_dict = {}
        right_side_dict = {}
        y_hist = {}
        root_list = []
        main_lines = []
        lines_dist = []

        dist = []
        for i, row in result_df.iterrows():
            if row['dependency_type'].lower().strip() != 'root':
                lines_dist.append(abs(int(row['begin']) - int(row['dependency_start']['head.begin'])))
            else:
                lines_dist.append(0)
            
        result_df = result_df.iloc[np.argsort(lines_dist)]

        count_left = {}
        count_right = {}
        t_x_offset = {}
        for i, row in result_df.iterrows():
          if row['dependency_type'].lower().strip() != 'root':
            sp = str(row['dependency_start']['head.begin'])+str(row['dependency_start']['head.end'])
            x_e, y_e = tk_dict[str(row['begin'])+str(row['end'])]
            x, y = tk_dict[sp]
            if int(row['begin']) < int(row['dependency_start']['head.begin']):
              if x in count_left:
                count_left[x] += 1
                t_x_offset[x] += 7
              else:
                count_left[x] = 1
                t_x_offset[x] = 7
              if x_e in count_right:
                count_right[x_e] += 1
                t_x_offset[x_e] -= 7
              else:
                count_right[x_e] = 0
                t_x_offset[x_e] = 0
            else:
              if x in count_right:
                count_right[x] += 1
                t_x_offset[x] -= 7
              else:
                count_right[x] = 0
                t_x_offset[x] = 0
              if x_e in count_left:
                count_left[x_e] += 1
                t_x_offset[x_e] += 7
              else:
                count_left[x_e] = 1
                t_x_offset[x_e] = 7
        
        for i, row in result_df.iterrows():
          
          sp = str(row['dependency_start']['head.begin'])+str(row['dependency_start']['head.end'])
          ep = tk_dict[str(row['begin'])+str(row['end'])]

          if sp != '-1-1':
            x, y = tk_dict[sp]

            if int(row['begin']) > int(row['dependency_start']['head.begin']):
              dist_dict[x] = count_right[x] * 7
              count_right[x] -= 1
              e_dist_dict[ep[0]] = count_left[ep[0]] * -7
              count_left[ep[0]] -= 1
            else:
              dist_dict[x] = count_left[x] * -7
              count_left[x] -= 1
              e_dist_dict[ep[0]] = count_right[ep[0]] * 7
              count_right[ep[0]] -= 1
            #row['dependency'], x, t_x_offset[x], x+dist_dict[x], x+dist_dict[x]+t_x_offset[x]
            final_x_s = int(x+dist_dict[x]+(t_x_offset[x]/2))
            final_x_e = int(ep[0]+ e_dist_dict[ep[0]]+(t_x_offset[ep[0]]/2))

            x_inds = range(min(final_x_s, final_x_e), max(final_x_s, final_x_e)+1)
            common = set(y_hist.keys()).intersection(set(x_inds))

            if common:
              y_fset = min([y_hist[c] for c in common])
              y_fset -= 50
              y_hist.update(dict(zip(x_inds, [y_fset]*len(x_inds))))
              
            else:
              y_hist.update(dict(zip(x_inds, [y_offset]*len(x_inds))))

            main_lines.append((None, final_x_s, y-30, final_x_e, y_hist[final_x_s], row['dependency_type']))

          else:
            x_x , y_y = tk_dict[str(row['begin'])+str(row['end'])]

            root_list.append((row['dependency_type'].upper(), x_x, y_y))


        current_y = min(y_hist.values())

        y_ff = (max_y - current_y) + 50
        y_f = (current_y - 50)
        current_y = 50

        dwg = svgwrite.Drawing("temp.svg",
                              profile='tiny', size = (max_x, y_ff+100))

        for mt, mp in zip(main_text, main_pos):
          dwg.add(dwg.text(mt[0], insert=(mt[1], mt[2]-y_f), fill='gray', 
          font_size='25', font_family='courier'))

          if mt[3]:
            dwg.add(dwg.rect(insert=(mt[1]-5, mt[2]-y_f-25), size=(self.__size(mt[0]),35), stroke='orange', 
            stroke_width='2', fill='none'))

          dwg.add(dwg.text(mp[0], insert=(mp[1], mp[2]-y_f), fill=colors_dict[mp[0]]))

        for ml in main_lines:
          self.__draw_line(dwg, ml[1], ml[2]-y_f, ml[3], ml[4]-y_f, ml[5], colors_dict[ml[5]])
        
        return dwg.tostring()


    def display(self, res, pos_col, dependency_col, dependency_type_col):
        """Displays NER visualization. 

        Inputs:
        result -- A Dataframe or dictionary.
        label_col -- Name of the column/key containing NER annotations.
        document_col -- Name of the column/key containing text document.
        original_text -- Original text of type 'str'. If specified, it will take precedence over 'document_col' and will be used as the reference text for display.
        labels_list -- A list of labels that should be highlighted in the output. Default: Display all labels.

        Output: Visualization
        """

        pos_res = []
        for i in res[pos_col]:
            t_ = {'chunk': i.metadata['word'],
            'begin': i.begin,
            'end' : i.end,
            'pos' : i.result}
            pos_res.append(t_)
        dep_res = []
        dep_res_meta = []
        for i in res[dependency_col]:
            dep_res.append(i.result)
            dep_res_meta.append(i.metadata)
        df = pd.DataFrame(pos_res)
        df['dependency'] = dep_res
        df['dependency_start'] = dep_res_meta

        dept_res = []
        for i in res[dependency_type_col]:
            dept_res.append(i.result)
        df['dependency_type'] = dept_res

        return display(HTML(self.__generate_graph(df)))




import setuptools
import os

here = os.path.abspath(os.path.dirname(__file__))
with open(os.path.join(here, "README.md"), "r") as fh:
    long_description = fh.read()

with open(os.path.join(here, "sparknlp_display/VERSION"), "r") as fh:
    app_version = fh.read().strip()

setuptools.setup(
    name="spark-nlp-display",
    version=app_version,
    author="John Snow Labs",
    author_email="john@johnsnowlabs.com",
    description="Visualization package for Spark NLP",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="http://nlp.johnsnowlabs.com",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 2",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=2.7',
    include_package_data=True,
    install_requires=[
        'spark-nlp',
        'ipython',
        'svgwrite==1.4',
        'pandas',
        'numpy'
    ]
)